@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Edgewall Software
+# Copyright (C) 2018-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -13,6 +13,7 @@
 # history and logs, available at https://trac.edgewall.org/log/.
 
 from datetime import datetime
+import argparse
 import os.path
 import sys
 
@@ -20,7 +21,7 @@ from babel.core import negotiate_locale
 from babel.messages.pofile import read_po, write_po
 
 
-class ScriptError(StandardError):
+class ScriptError(Exception):
     pass
 
 
@@ -36,8 +37,11 @@ def _has_msgstr(message):
 
 
 def _open_pofile(filename):
-    with open(filename, 'rb') as f:
-        return read_po(f)
+    try:
+        with open(filename, 'rb') as f:
+            return read_po(f)
+    except IOError as e:
+        raise ScriptError(e)
 
 
 def _get_domains():
@@ -54,18 +58,25 @@ def _get_locales(domain):
                        if has_pofile(name))
 
 
-def main(args):
-    if len(args) not in (2, 3):
-        raise ScriptError('Usage: %s DOMAIN POFILE [LOCALE]' %
-                          os.path.basename(sys.argv[0]))
-    domain, source_file, locale = (args + [None])[:3]
+def main():
+    """Merge translated strings from another PO file.
+
+    $ src=../trac-1.2-stable/trac/locale/de/LC_MESSAGES
+    $ PYTHONPATH=. contrib/%(prog)s messages $src/messages.po [locale]
+    """
     domains = _get_domains()
-    locales = _get_locales(domain)
     if not domains:
         raise ScriptError('No trac/locale/*.pot files.')
-    if domain not in domains:
-        raise ScriptError('Domain parameter should be one of %s.' %
-                          ', '.join(domains))
+    parser = argparse.ArgumentParser(usage=main.__doc__)
+    parser.add_argument('domain', choices=domains,
+                        help="Name of catalog to merge")
+    parser.add_argument('pofile', help="Path of the catalog to merge from")
+    parser.add_argument('locale', help="Locale of the catalog to merge from",
+                        nargs='?', default=None)
+    args = parser.parse_args()
+
+    domain, source_file, locale = args.domain, args.pofile, args.locale
+    locales = _get_locales(domain)
     if not locales:
         raise ScriptError('No trac/locale/*/LC_MESSAGES/*.po files.')
     pot = _open_pofile(os.path.join('trac', 'locale', domain + '.pot'))
@@ -126,9 +137,10 @@ def main(args):
 
 
 if __name__ == '__main__':
+    rc = 0
     try:
-        rc = main(sys.argv[1:])
+        main()
     except ScriptError as e:
         rc = 1
         sys.stderr.write('%s\n' % e)
-    sys.exit(rc or 0)
+    sys.exit(rc)
