@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2020 Edgewall Software
+# Copyright (C) 2004-2021 Edgewall Software
 # Copyright (C) 2004 Dmitry Yusupov <dmitry_yus@yahoo.com>
 # Copyright (C) 2004 Mark Rowe <mrowe@bluewire.net.nz>
 # Copyright (C) 2010 Anatoly Techtonik <techtonik@php.net>
@@ -78,14 +78,14 @@ user nobody == anonymous
 Not implemented (feature:reason)
 --------------------------------
 attachments:made as a comment with links to attachments stored on SF
-            (type,id,filename,size,time,description,author,ipnr)
+            (type,id,filename,size,time,description,author)
 ticket_custom:unknown (ticket,name,value)
 history:imported only for summary, priority. closed date and owner fields
 
 severities:no field in source data
 """
 
-import optparse
+import argparse
 import sys
 import time
 from xml.etree.ElementTree import ElementTree
@@ -128,7 +128,7 @@ class FlatXML(object):
         """merge supplied ElementTree element into current object"""
         for c in el:
             if len(c.getchildren()) == 0:
-                if c.text != None and len(c.text.strip()) != 0:
+                if c.text is not None and len(c.text.strip()) != 0:
                     self.__setattr__(c.tag, c.text)
                 else:
                     self.__setattr__(c.tag, [])
@@ -315,8 +315,8 @@ class ExportedProjectData(object):
 
         root = ElementTree().parse(f)
 
-        self.users = dict([(FlatXML(u).userid, FlatXML(u).username)
-                          for u in root.find('referenced_users')])
+        self.users = {FlatXML(u).userid: FlatXML(u).username
+                      for u in root.find('referenced_users')}
 
         for tracker in root.find('trackers'):
             tr = Tracker(tracker)
@@ -512,7 +512,7 @@ class TracDatabase(object):
                   """, (ticket, time, author, field, oldvalue, newvalue))
 
 
-def importData(f, env, opt):
+def importData(f, env):
     project = ExportedProjectData(f)
     trackers = project.trackers
 
@@ -520,8 +520,8 @@ def importData(f, env, opt):
 
     # Data conversion
     typeList = [x.name for x in trackers]
-    print("%d trackers will be converted to the following ticket types:\n  %s" \
-        % (len(trackers), typeList))
+    print("%d trackers will be converted to the following ticket types:\n  %s"
+          % (len(trackers), typeList))
 
     used_cat_names = set(project.used_categories.values())
     #: make names unique, forget about competing owners (the last one wins)
@@ -553,9 +553,9 @@ def importData(f, env, opt):
     # Data save
     trac.setTypeList(typeList)
     trac.setComponentList(components)
-    trac.setPriorityList(range(min(project.priorities),
-                               max(project.priorities)))
-    trac.setVersionList(set([x[1] for x in project.groups]))
+    trac.setPriorityList(xrange(min(project.priorities),
+                                max(project.priorities)))
+    trac.setVersionList({x[1] for x in project.groups})
     trac.setResolutionList(resolutions)
     trac.setMilestoneList([])
 
@@ -695,15 +695,15 @@ def importData(f, env, opt):
 
 
 def main():
-    p = optparse.OptionParser(
-            "Usage: %prog xml_export.xml /path/to/trac/environment")
-    opt, args = p.parse_args()
-    if len(args) != 2:
-        p.error("Incorrect number of arguments")
+    parser = argparse.ArgumentParser()
+    parser.add_argument('filename', help="XML export file from sourceforge")
+    parser.add_argument('environment', help="path to Trac environment")
+    args = parser.parse_args()
 
     try:
-        importData(open(args[0]), args[1], opt)
-    except DBNotEmpty as e:
+        with open(args.filename, 'r') as f:
+            importData(f, args.environment)
+    except Exception as e:
         printerr("Error: %s" % e)
         sys.exit(1)
 
