@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2020 Edgewall Software
+# Copyright (C) 2009-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -138,8 +138,8 @@ def cached(fn_or_attr=None):
         class WikiSystem(Component):
             @cached
             def pages(self):
-                return set(name for name, in self.env.db_query(
-                               "SELECT DISTINCT name FROM wiki"))
+                return {name for name, in self.env.db_query(
+                            "SELECT DISTINCT name FROM wiki")}
 
     Otherwise, when the decorator is used in non-"singleton" objects,
     a string specifying the name of an attribute containing a string
