@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -14,97 +14,25 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+import importlib
 import os
 import time
 import urllib
 from abc import ABCMeta, abstractmethod
 
-from genshi.builder import tag
+from trac import db_default
+from trac.api import IEnvironmentSetupParticipant, ISystemInfoProvider
 from trac.config import BoolOption, ConfigurationError, IntOption, Option
 from trac.core import *
 from trac.db.pool import ConnectionPool
 from trac.db.schema import Table
 from trac.db.util import ConnectionWrapper
 from trac.util.concurrency import ThreadLocal
+from trac.util.html import tag
 from trac.util.text import unicode_passwd
 from trac.util.translation import _, tag_
 
 
-def with_transaction(env, db=None):
-    """Function decorator to emulate a context manager for database
-    transactions.
-
-    >>> def api_method(p1, p2):
-    >>>     result[0] = value1
-    >>>     @with_transaction(env)
-    >>>     def implementation(db):
-    >>>         # implementation
-    >>>         result[0] = value2
-    >>>     return result[0]
-
-    In this example, the `implementation()` function is called
-    automatically right after its definition, with a database
-    connection as an argument. If the function completes, a COMMIT is
-    issued on the connection. If the function raises an exception, a
-    ROLLBACK is issued and the exception is re-raised. Nested
-    transactions are supported, and a COMMIT will only be issued when
-    the outermost transaction block in a thread exits.
-
-    This mechanism is intended to replace the former practice of
-    getting a database connection with `env.get_db_cnx()` and issuing
-    an explicit commit or rollback, for mutating database
-    accesses. Its automatic handling of commit, rollback and nesting
-    makes it much more robust.
-
-    The optional `db` argument is intended for legacy code and should
-    not be used in new code.
-
-    :deprecated: This decorator is in turn deprecated in favor of
-                 context managers now that python 2.4 support has been
-                 dropped. It will be removed in Trac 1.3.1. Use instead
-                 the new context managers, `QueryContextManager` and
-                 `TransactionContextManager`, which make for much
-                 simpler to write code:
-
-    >>> def api_method(p1, p2):
-    >>>     result = value1
-    >>>     with env.db_transaction as db:
-    >>>         # implementation
-    >>>         result = value2
-    >>>     return result
-
-    """
-    dbm = DatabaseManager(env)
-    _transaction_local = dbm._transaction_local
-
-    def transaction_wrapper(fn):
-        ldb = _transaction_local.wdb
-        if db is not None:
-            if ldb is None:
-                _transaction_local.wdb = db
-                try:
-                    fn(db)
-                finally:
-                    _transaction_local.wdb = None
-            else:
-                assert ldb is db, "Invalid transaction nesting"
-                fn(db)
-        elif ldb:
-            fn(ldb)
-        else:
-            ldb = _transaction_local.wdb = dbm.get_connection()
-            try:
-                fn(ldb)
-                ldb.commit()
-                _transaction_local.wdb = None
-            except:
-                _transaction_local.wdb = None
-                ldb.rollback()
-                ldb = None
-                raise
-    return transaction_wrapper
-
-
 class DbContextManager(object):
     """Database Context Manager
 
@@ -138,9 +66,9 @@ class TransactionContextManager(DbContextManager):
     """
 
     def __enter__(self):
-        db = self.dbmgr._transaction_local.wdb # outermost writable db
+        db = self.dbmgr._transaction_local.wdb  # outermost writable db
         if not db:
-            db = self.dbmgr._transaction_local.rdb # reuse wrapped connection
+            db = self.dbmgr._transaction_local.rdb  # reuse wrapped connection
             if db:
                 db = ConnectionWrapper(db.cnx, db.log)
             else:
@@ -165,9 +93,9 @@ class QueryContextManager(DbContextManager):
     """
 
     def __enter__(self):
-        db = self.dbmgr._transaction_local.rdb # outermost readonly db
+        db = self.dbmgr._transaction_local.rdb  # outermost readonly db
         if not db:
-            db = self.dbmgr._transaction_local.wdb # reuse wrapped connection
+            db = self.dbmgr._transaction_local.wdb  # reuse wrapped connection
             if db:
                 db = ConnectionWrapper(db.cnx, db.log, readonly=True)
             else:
@@ -219,11 +147,21 @@ class ConnectionBase(object):
         to `id`."""
         pass
 
+    @abstractmethod
+    def get_sequence_names(self):
+        """Returns a list of the sequence names."""
+        pass
+
     @abstractmethod
     def get_table_names(self):
         """Returns a list of the table names."""
         pass
 
+    @abstractmethod
+    def has_table(self, table):
+        """Returns whether the table exists."""
+        pass
+
     @abstractmethod
     def like(self):
         """Returns a case-insensitive `LIKE` clause."""
@@ -307,10 +245,18 @@ class IDatabaseConnector(Interface):
         """Backup the database to a location defined by
         trac.backup_dir"""
 
+    def get_system_info():
+        """Yield a sequence of `(name, version)` tuples describing the
+        name and version information of external packages used by the
+        connector.
+        """
+
 
 class DatabaseManager(Component):
     """Component used to manage the `IDatabaseConnector` implementations."""
 
+    implements(IEnvironmentSetupParticipant, ISystemInfoProvider)
+
     connectors = ExtensionPoint(IDatabaseConnector)
 
     connection_uri = Option('trac', 'database', 'sqlite:db/trac.db',
@@ -335,8 +281,7 @@ class DatabaseManager(Component):
 
     def init_db(self):
         connector, args = self.get_connector()
-        from trac.db_default import schema
-        args['schema'] = schema
+        args['schema'] = db_default.schema
         connector.init_db(**args)
 
     def destroy_db(self):
@@ -367,9 +312,13 @@ class DatabaseManager(Component):
 
         :since: version 1.2
         """
+        table_name = table.name if isinstance(table, Table) else table
         with self.env.db_transaction as db:
+            if not db.has_table(table_name):
+                raise self.env.db_exc.OperationalError('Table %s not found' %
+                                                       db.quote(table_name))
             for col in columns:
-                db.drop_column(table, col)
+                db.drop_column(table_name, col)
 
     def drop_tables(self, schema):
         """Drop the specified tables.
@@ -430,19 +379,21 @@ class DatabaseManager(Component):
             cursor = db.cursor()
             for new_table in new_schema:
                 temp_table_name = new_table.name + '_old'
-                old_column_names = set(self.get_column_names(new_table))
-                new_column_names = set(col.name for col in new_table.columns)
-                column_names = old_column_names & new_column_names
-                cols_to_copy = ','.join(db.quote(name)
-                                        for name in column_names)
-                if cols_to_copy:
-                    cursor.execute("""
-                        CREATE TEMPORARY TABLE %s AS SELECT * FROM %s
-                        """ % (db.quote(temp_table_name),
-                               db.quote(new_table.name)))
-                self.drop_tables((new_table,))
+                has_table = self.has_table(new_table)
+                if has_table:
+                    old_column_names = set(self.get_column_names(new_table))
+                    new_column_names = {col.name for col in new_table.columns}
+                    column_names = old_column_names & new_column_names
+                    if column_names:
+                        cols_to_copy = ','.join(db.quote(name)
+                                                for name in column_names)
+                        cursor.execute("""
+                            CREATE TEMPORARY TABLE %s AS SELECT * FROM %s
+                            """ % (db.quote(temp_table_name),
+                                   db.quote(new_table.name)))
+                    self.drop_tables((new_table,))
                 self.create_tables((new_table,))
-                if cols_to_copy:
+                if has_table and column_names:
                     cursor.execute("""
                         INSERT INTO %s (%s) SELECT %s FROM %s
                         """ % (db.quote(new_table.name), cols_to_copy,
@@ -476,14 +427,24 @@ class DatabaseManager(Component):
                      which contains the database version for Trac.
         """
         with self.env.db_query as db:
-            rows = db("""
-                SELECT value FROM {0} WHERE name=%s
-                """.format(db.quote('system')), (name,))
-        return int(rows[0][0]) if rows else False
+            for value, in db("""
+                    SELECT value FROM {0} WHERE name=%s
+                    """.format(db.quote('system')), (name,)):
+                return int(value)
+            else:
+                return False
 
     def get_exceptions(self):
         return self.get_connector()[0].get_exceptions()
 
+    def get_sequence_names(self):
+        """Returns a list of the sequence names.
+
+        :since: 1.3.2
+        """
+        with self.env.db_query as db:
+            return db.get_sequence_names()
+
     def get_table_names(self):
         """Returns a list of the table names.
 
@@ -495,14 +456,23 @@ class DatabaseManager(Component):
     def get_column_names(self, table):
         """Returns a list of the column names for `table`.
 
-        :param schema: a `Table` object or table name.
+        :param table: a `Table` object or table name.
 
         :since: 1.2
         """
         table_name = table.name if isinstance(table, Table) else table
         with self.env.db_query as db:
+            if not db.has_table(table_name):
+                raise self.env.db_exc.OperationalError('Table %s not found' %
+                                                       db.quote(table_name))
             return db.get_column_names(table_name)
 
+    def has_table(self, table):
+        """Returns whether the table exists."""
+        table_name = table.name if isinstance(table, Table) else table
+        with self.env.db_query as db:
+            return db.has_table(table_name)
+
     def set_database_version(self, version, name='database_version'):
         """Sets the database version in the SYSTEM table.
 
@@ -560,20 +530,16 @@ class DatabaseManager(Component):
         :raises TracError: if the package or module doesn't exist.
         """
         dbver = self.get_database_version(name)
-        for i in range(dbver + 1, version + 1):
-            module = 'db%i' % i
+        for i in xrange(dbver + 1, version + 1):
+            module = '%s.db%i' % (pkg, i)
             try:
-                upgrades = __import__(pkg, globals(), locals(), [module])
+                upgrader = importlib.import_module(module)
             except ImportError:
-                raise TracError(_("No upgrade package %(pkg)s", pkg=pkg))
-            try:
-                script = getattr(upgrades, module)
-            except AttributeError:
                 raise TracError(_("No upgrade module %(module)s.py",
                                   module=module))
             with self.env.db_transaction as db:
                 cursor = db.cursor()
-                script.do_upgrade(self.env, i, cursor)
+                upgrader.do_upgrade(self.env, i, cursor)
                 self.set_database_version(i, name)
 
     def shutdown(self, tid=None):
@@ -635,7 +601,24 @@ class DatabaseManager(Component):
             args['log'] = self.log
         return connector, args
 
-    _get_connector = get_connector  # For 0.11 compatibility
+    # IEnvironmentSetupParticipant methods
+
+    def environment_created(self):
+        """Insert default data into the database."""
+        self.insert_into_tables(db_default.get_data)
+
+    def environment_needs_upgrade(self):
+        return self.needs_upgrade(db_default.db_version)
+
+    def upgrade_environment(self):
+        self.upgrade(db_default.db_version)
+
+    # ISystemInfoProvider methods
+
+    def get_system_info(self):
+        connector = self.get_connector()[0]
+        for info in connector.get_system_info():
+            yield info
 
 
 def get_column_names(cursor):
@@ -738,11 +721,7 @@ def parse_connection_uri(db_str):
 
     args = zip(('user', 'password', 'host', 'port', 'path', 'params'),
                (user, password, host, port, path, params))
-    return scheme, dict([(key, value) for key, value in args if value])
-
-
-# Compatibility for Trac < 1.1.3. Will be removed in 1.3.1.
-_parse_db_str = parse_connection_uri
+    return scheme, {key: value for key, value in args if value}
 
 
 def _invalid_db_str(db_str):
