@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2005 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # Copyright (C) 2008 Stephen Hansen
@@ -27,8 +27,6 @@ from email.mime.text import MIMEText
 from email.utils import formatdate, parseaddr, getaddresses
 from subprocess import Popen, PIPE
 
-from genshi.builder import tag
-
 from trac.config import (BoolOption, ConfigurationError, IntOption, Option,
                          OrderedExtensionsOption)
 from trac.core import Component, ExtensionPoint, TracError, implements
@@ -39,6 +37,7 @@ from trac.notification.api import (
 from trac.util import lazy
 from trac.util.compat import close_fds
 from trac.util.datefmt import time_now, to_utimestamp
+from trac.util.html import tag
 from trac.util.text import CRLF, exception_to_unicode, fix_eol, to_unicode
 from trac.util.translation import _, tag_
 from trac.web.session import get_session_attribute
@@ -70,7 +69,7 @@ def create_charset(mime_encoding):
     """Create an appropriate email charset for the given encoding.
 
     Valid options are 'base64' for Base64 encoding, 'qp' for
-    Quoted-Printable, and 'none' for no encoding, in which case mails
+    Quoted-Printable, and 'none' for no encoding, in which case emails
     will be sent as 7bit if the content is all ASCII, or 8bit otherwise.
     """
     charset = Charset()
@@ -106,7 +105,7 @@ def create_header(key, value, charset):
     message.
     """
     maxlength = MAXHEADERLEN-(len(key)+2)
-    # Do not sent ridiculous short headers
+    # Do not sent very short headers
     if maxlength < 10:
         raise TracError(_("Header length is too short"))
 
@@ -399,7 +398,7 @@ class EmailDistributor(Component):
         'SessionEmailResolver',
         include_missing=False,
         doc="""Comma separated list of email resolver components in the order
-        they will be called.  If an email address is resolved, the remaining
+        they will be called. If an email address is resolved, the remaining
         resolvers will not be called.
         """)
 
@@ -434,7 +433,7 @@ class EmailDistributor(Component):
             return
         self.log.debug("%s has found the following formats capable of "
                        "handling '%s' of '%s': %s", self.__class__.__name__,
-                       transport, event.realm, ', '.join(formats.keys()))
+                       transport, event.realm, ', '.join(formats))
 
         matcher = RecipientMatcher(self.env)
         notify_sys = NotificationSystem(self.env)
@@ -487,11 +486,11 @@ class EmailDistributor(Component):
             try:
                 outputs[fmt] = formatter.format(transport, fmt, event)
             except Exception as e:
-                self.log.warn('%s caught exception while '
-                              'formatting %s to %s for %s: %s%s',
-                              self.__class__.__name__, event.realm, fmt,
-                              transport, formatter.__class__,
-                              exception_to_unicode(e, traceback=True))
+                self.log.warning('%s caught exception while '
+                                 'formatting %s to %s for %s: %s%s',
+                                 self.__class__.__name__, event.realm, fmt,
+                                 transport, formatter.__class__,
+                                 exception_to_unicode(e, traceback=True))
                 failed.append(fmt)
 
         # Fallback to text/plain when formatter is broken
@@ -510,9 +509,9 @@ class EmailDistributor(Component):
                 bcc_addrs = sorted(addrs - always_cc)
                 self._do_send(transport, event, message, cc_addrs, bcc_addrs)
             else:
-                self.log.warn("%s cannot send event '%s' as '%s': %s",
-                              self.__class__.__name__, event.realm, fmt,
-                              ', '.join(addrs))
+                self.log.warning("%s cannot send event '%s' as '%s': %s",
+                                 self.__class__.__name__, event.realm, fmt,
+                                 ', '.join(addrs))
 
     def _create_message(self, format, outputs):
         if format not in outputs:
@@ -667,7 +666,7 @@ class SendmailEmailSender(Component):
         """Path to the sendmail executable.
 
         The sendmail program must accept the `-i` and `-f` options.
-         (''since 0.12'')""")
+        """)
 
     def send(self, from_addr, recipients, message):
         # Use native line endings in message
