@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2020 Edgewall Software
+# Copyright (C) 2004-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -231,6 +231,13 @@ class PermissionSystemTestCase(BaseTestCase):
         self.assertEqual(tpr_perms,
                          self.perm.get_actions(skip=self.perm))
 
+    def test_actions(self):
+        self.assertEqual(self.perm.get_actions(), self.perm.actions)
+
+    def test_actions_is_lazy(self):
+        actions = self.perm.actions
+        self.assertEqual(id(actions), id(self.perm.actions))
+
     def test_get_actions_dict(self):
         self.assertEqual({
             'TEST_ADMIN': ['TEST_CREATE', 'TEST_DELETE', 'TEST_MODIFY'],
