@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -19,7 +19,7 @@ from trac.resource import ResourceNotFound
 from trac.ticket import model
 from trac.ticket.api import TicketSystem
 from trac.ticket.roadmap import MilestoneModule
-from trac.util import getuser
+from trac.util import as_int, getuser
 from trac.util.datefmt import format_date, format_datetime, \
                               get_datetime_format_hint, parse_date, user_time
 from trac.util.text import exception_to_unicode, print_table, printout
@@ -35,7 +35,6 @@ class TicketAdminPanel(Component):
 
     _type = 'undefined'
     _label = N_("(Undefined)"), N_("(Undefined)")
-    _view_perms = ['TICKET_ADMIN']
 
     # i18n note: use gettext() whenever referring to the above as text labels,
     #            and don't use it whenever using them as field names (after
@@ -44,8 +43,7 @@ class TicketAdminPanel(Component):
     # IAdminPanelProvider methods
 
     def get_admin_panels(self, req):
-        if all(perm in req.perm('admin', 'ticket/' + self._type)
-               for perm in self._view_perms):
+        if 'TICKET_ADMIN' in req.perm('admin', 'ticket/' + self._type):
             yield ('ticket', _('Ticket System'), self._type,
                    gettext(self._label[1]))
 
@@ -71,6 +69,10 @@ class TicketAdminPanel(Component):
         else:
             add_notice(req, _("Your changes have been saved."))
 
+    def _render_admin_panel(self, req, cat, page, path_info):
+        raise NotImplemented("Class inheriting from TicketAdminPanel has not "
+                             "implemented the _render_admin_panel method.")
+
 
 class ComponentAdminPanel(TicketAdminPanel):
 
@@ -85,20 +87,18 @@ class ComponentAdminPanel(TicketAdminPanel):
             comp = model.Component(self.env, component)
             if req.method == 'POST':
                 if req.args.get('save'):
-                    comp.name = name = req.args.get('name')
+                    comp.name = req.args.get('name')
                     comp.owner = req.args.get('owner')
                     comp.description = req.args.get('description')
-                    try:
-                        comp.update()
-                    except self.env.db_exc.IntegrityError:
-                        raise TracError(_('Component "%(name)s" already '
-                                          'exists.', name=comp.name))
+                    comp.update()
                     add_notice(req, _("Your changes have been saved."))
                     req.redirect(req.href.admin(cat, page))
                 elif req.args.get('cancel'):
                     req.redirect(req.href.admin(cat, page))
 
-            Chrome(self.env).add_wiki_toolbars(req)
+            chrome = Chrome(self.env)
+            chrome.add_wiki_toolbars(req)
+            chrome.add_auto_preview(req)
             data = {'view': 'detail', 'component': comp}
 
         else:
@@ -106,26 +106,12 @@ class ComponentAdminPanel(TicketAdminPanel):
             if req.method == 'POST':
                 # Add Component
                 if req.args.get('add') and req.args.get('name'):
-                    name = req.args.get('name')
-                    try:
-                        comp = model.Component(self.env, name=name)
-                    except ResourceNotFound:
-                        comp = model.Component(self.env)
-                        comp.name = name
-                        if req.args.get('owner'):
-                            comp.owner = req.args.get('owner')
-                        try:
-                            comp.insert()
-                        except self.env.db_exc.IntegrityError:
-                            raise TracError(_('Component "%(name)s" already '
-                                              'exists.', name=comp.name))
-                        add_notice(req, _('The component "%(name)s" has been '
-                                          'added.', name=comp.name))
-                    else:
-                        if comp.name is None:
-                            raise TracError(_("Invalid component name."))
-                        raise TracError(_('Component "%(name)s" already '
-                                          'exists.', name=name))
+                    comp = model.Component(self.env)
+                    comp.name = req.args.get('name')
+                    comp.owner = req.args.get('owner')
+                    comp.insert()
+                    add_notice(req, _('The component "%(name)s" has been '
+                                      'added.', name=comp.name))
 
                 # Remove components
                 elif req.args.get('remove'):
@@ -155,7 +141,6 @@ class ComponentAdminPanel(TicketAdminPanel):
                     self.log.info("Clearing default component")
                     self.config.set('ticket', 'default_component', '')
                     self._save_config(req)
-                    req.redirect(req.href.admin(cat, page))
 
                 req.redirect(req.href.admin(cat, page))
 
@@ -174,19 +159,19 @@ class ComponentAdminPanel(TicketAdminPanel):
 
     def get_admin_commands(self):
         yield ('component list', '',
-               'Show available components',
+               "Show components",
                None, self._do_list)
         yield ('component add', '<name> [owner]',
-               'Add a new component',
+               "Add component",
                self._complete_add, self._do_add)
         yield ('component rename', '<name> <newname>',
-               'Rename a component',
-               self._complete_remove_rename, self._do_rename)
+               "Rename component",
+               self._complete_name, self._do_rename)
         yield ('component remove', '<name>',
-               'Remove/uninstall a component',
-               self._complete_remove_rename, self._do_remove)
+               "Remove component",
+               self._complete_name, self._do_remove)
         yield ('component chown', '<name> <owner>',
-               'Change component ownership',
+               "Change component owner",
                self._complete_chown, self._do_chown)
 
     def get_component_list(self):
@@ -199,7 +184,7 @@ class ComponentAdminPanel(TicketAdminPanel):
         if len(args) == 2:
             return self.get_user_list()
 
-    def _complete_remove_rename(self, args):
+    def _complete_name(self, args):
         if len(args) == 1:
             return self.get_component_list()
 
@@ -212,7 +197,7 @@ class ComponentAdminPanel(TicketAdminPanel):
     def _do_list(self):
         print_table([(c.name, c.owner)
                      for c in model.Component.select(self.env)],
-                    [_('Name'), _('Owner')])
+                    [_("Name"), _("Owner")])
 
     def _do_add(self, name, owner=None):
         component = model.Component(self.env)
@@ -238,10 +223,16 @@ class MilestoneAdminPanel(TicketAdminPanel):
 
     _type = 'milestones'
     _label = N_("Milestone"), N_("Milestones")
-    _view_perms = TicketAdminPanel._view_perms + ['MILESTONE_VIEW']
 
     # TicketAdminPanel methods
 
+    def get_admin_panels(self, req):
+        perm = req.perm('admin', 'ticket/' + self._type)
+        if 'MILESTONE_ADMIN' in perm or \
+                'MILESTONE_VIEW' in perm and 'TICKET_ADMIN' in perm:
+            yield ('ticket', _('Ticket System'), self._type,
+                   gettext(self._label[1]))
+
     def _render_admin_panel(self, req, cat, page, milestone_name):
         perm_cache = req.perm('admin', 'ticket/' + self._type)
 
@@ -258,7 +249,9 @@ class MilestoneAdminPanel(TicketAdminPanel):
                 elif 'cancel' in req.args:
                     req.redirect(req.href.admin(cat, page))
 
-            Chrome(self.env).add_wiki_toolbars(req)
+            chrome = Chrome(self.env)
+            chrome.add_wiki_toolbars(req)
+            chrome.add_auto_preview(req)
             data = {'view': 'detail',
                     'milestone': milestone,
                     'default_due': milestone_module.get_default_due(req)}
@@ -312,9 +305,10 @@ class MilestoneAdminPanel(TicketAdminPanel):
                     add_notice(req, _("The selected milestones have been "
                                       "removed."))
 
-                # Set default milestone
+                # Set default ticket milestone and retarget milestone
                 elif 'apply' in req.args:
                     save = False
+                    perm_cache.require('TICKET_ADMIN')
                     name = req.args.get('ticket_default')
                     if name and name != ticket_default:
                         self.log.info("Setting default ticket "
@@ -331,14 +325,14 @@ class MilestoneAdminPanel(TicketAdminPanel):
                     if save:
                         self._save_config(req)
 
-                # Clear default milestone
+                # Clear default ticket milestone and retarget milestone
                 elif 'clear' in req.args:
+                    perm_cache.require('TICKET_ADMIN')
                     self.log.info("Clearing default ticket milestone "
                                   "and default retarget milestone")
                     self.config.set('ticket', 'default_milestone', '')
                     self.config.set('milestone', 'default_retarget_to', '')
                     self._save_config(req)
-                    req.redirect(req.href.admin(cat, page))
 
                 req.redirect(req.href.admin(cat, page))
 
@@ -354,8 +348,7 @@ class MilestoneAdminPanel(TicketAdminPanel):
             data = {'view': 'list',
                     'milestones': model.Milestone.select(self.env),
                     'query_href': query_href,
-                    'num_tickets': lambda milestone:
-                                            num_tickets.get(milestone.name, 0),
+                    'num_tickets': lambda m: num_tickets.get(m.name, 0),
                     'ticket_default': ticket_default,
                     'retarget_default': retarget_default}
 
@@ -451,7 +444,7 @@ class MilestoneAdminPanel(TicketAdminPanel):
         milestone.update()
 
     def _do_remove(self, name):
-        model.Milestone(self.env, name).delete(author=getuser())
+        model.Milestone(self.env, name).delete()
 
 
 class VersionAdminPanel(TicketAdminPanel):
@@ -467,26 +460,18 @@ class VersionAdminPanel(TicketAdminPanel):
             ver = model.Version(self.env, version)
             if req.method == 'POST':
                 if req.args.get('save'):
-                    ver.name = name = req.args.get('name')
-                    if req.args.get('time'):
-                        ver.time = user_time(req, parse_date,
-                                             req.args.get('time'),
-                                             hint='datetime')
-                    else:
-                        ver.time = None  # unset
+                    ver.name = req.args.get('name')
+                    ver.time = self._get_user_time(req)
                     ver.description = req.args.get('description')
-                    try:
-                        ver.update()
-                    except self.env.db_exc.IntegrityError:
-                        raise TracError(_('Version "%(name)s" already '
-                                          'exists.', name=ver.name))
-
+                    ver.update()
                     add_notice(req, _("Your changes have been saved."))
                     req.redirect(req.href.admin(cat, page))
                 elif req.args.get('cancel'):
                     req.redirect(req.href.admin(cat, page))
 
-            Chrome(self.env).add_wiki_toolbars(req)
+            chrome = Chrome(self.env)
+            chrome.add_wiki_toolbars(req)
+            chrome.add_auto_preview(req)
             data = {'view': 'detail', 'version': ver}
 
         else:
@@ -494,28 +479,12 @@ class VersionAdminPanel(TicketAdminPanel):
             if req.method == 'POST':
                 # Add Version
                 if req.args.get('add') and req.args.get('name'):
-                    name = req.args.get('name')
-                    try:
-                        ver = model.Version(self.env, name=name)
-                    except ResourceNotFound:
-                        ver = model.Version(self.env)
-                        ver.name = name
-                        if req.args.get('time'):
-                            ver.time = user_time(req, parse_date,
-                                                 req.args.get('time'),
-                                                 hint='datetime')
-                        try:
-                            ver.insert()
-                        except self.env.db_exc.IntegrityError:
-                            raise TracError(_('Version "%(name)s" already '
-                                              'exists.', name=ver.name))
-                        add_notice(req, _('The version "%(name)s" has been '
-                                          'added.', name=ver.name))
-                    else:
-                        if ver.name is None:
-                            raise TracError(_("Invalid version name."))
-                        raise TracError(_('Version "%(name)s" already '
-                                          'exists.', name=name))
+                    ver = model.Version(self.env)
+                    ver.name = req.args.get('name')
+                    ver.time = self._get_user_time(req)
+                    ver.insert()
+                    add_notice(req, _('The version "%(name)s" has been '
+                                      'added.', name=ver.name))
 
                 # Remove versions
                 elif req.args.get('remove'):
@@ -526,7 +495,8 @@ class VersionAdminPanel(TicketAdminPanel):
                         for name in sel:
                             model.Version(self.env, name).delete()
                             if name == default:
-                                self.config.set('ticket', 'default_version', '')
+                                self.config.set('ticket',
+                                                'default_version', '')
                                 self._save_config(req)
                     add_notice(req, _("The selected versions have been "
                                       "removed."))
@@ -544,7 +514,6 @@ class VersionAdminPanel(TicketAdminPanel):
                     self.log.info("Clearing default version")
                     self.config.set('ticket', 'default_version', '')
                     self._save_config(req)
-                    req.redirect(req.href.admin(cat, page))
 
                 req.redirect(req.href.admin(cat, page))
 
@@ -553,10 +522,16 @@ class VersionAdminPanel(TicketAdminPanel):
                     'default': default}
 
         Chrome(self.env).add_jquery_ui(req)
-
         data.update({'datetime_hint': get_datetime_format_hint(req.lc_time)})
+
         return 'admin_versions.html', data
 
+    @classmethod
+    def _get_user_time(cls, req):
+        time = req.args.get('time')
+        return user_time(req, parse_date, time, hint='datetime') \
+               if time else None
+
     # IAdminCommandProvider methods
 
     def get_admin_commands(self):
@@ -574,6 +549,9 @@ class VersionAdminPanel(TicketAdminPanel):
         yield ('version rename', '<name> <newname>',
                "Rename version",
                self._complete_name, self._do_rename)
+        yield ('version remove', '<name>',
+               "Remove version",
+               self._complete_name, self._do_remove)
         yield ('version time', '<name> <time>',
                """Set version date
 
@@ -584,9 +562,6 @@ class VersionAdminPanel(TicketAdminPanel):
                an empty string ("").
                """ % hints,
                self._complete_name, self._do_time)
-        yield ('version remove', '<name>',
-               "Remove version",
-               self._complete_name, self._do_remove)
 
     def get_version_list(self):
         return [v.name for v in model.Version.select(self.env)]
@@ -599,16 +574,15 @@ class VersionAdminPanel(TicketAdminPanel):
         print_table([(v.name,
                       format_date(v.time, console_date_format)
                       if v.time else None)
-                    for v in model.Version.select(self.env)],
+                     for v in model.Version.select(self.env)],
                     [_("Name"), _("Time")])
 
     def _do_add(self, name, time=None):
         version = model.Version(self.env)
         version.name = name
-        if time is not None:
-            version.time = parse_date(time, hint='datetime',
-                                      locale=get_console_locale(self.env)) \
-                           if time else None
+        version.time = parse_date(time, hint='datetime',
+                                  locale=get_console_locale(self.env)) \
+                       if time else None
         version.insert()
 
     def _do_rename(self, name, newname):
@@ -616,6 +590,9 @@ class VersionAdminPanel(TicketAdminPanel):
         version.name = newname
         version.update()
 
+    def _do_remove(self, name):
+        model.Version(self.env, name).delete()
+
     def _do_time(self, name, time):
         version = model.Version(self.env, name)
         version.time = parse_date(time, hint='datetime',
@@ -623,9 +600,6 @@ class VersionAdminPanel(TicketAdminPanel):
                        if time else None
         version.update()
 
-    def _do_remove(self, name):
-        model.Version(self.env, name).delete()
-
 
 class AbstractEnumAdminPanel(TicketAdminPanel):
 
@@ -646,17 +620,16 @@ class AbstractEnumAdminPanel(TicketAdminPanel):
             enum = self._enum_cls(self.env, path_info)
             if req.method == 'POST':
                 if req.args.get('save'):
-                    enum.name = name = req.args.get('name')
-                    try:
-                        enum.update()
-                    except self.env.db_exc.IntegrityError:
-                        raise TracError(_('%(type)s value "%(name)s" already '
-                                          'exists', type=label[0],
-                                          name=enum.name))
+                    enum.name = req.args.get('name')
+                    enum.description = req.args.get('description')
+                    enum.update()
                     add_notice(req, _("Your changes have been saved."))
                     req.redirect(req.href.admin(cat, page))
                 elif req.args.get('cancel'):
                     req.redirect(req.href.admin(cat, page))
+            chrome = Chrome(self.env)
+            chrome.add_wiki_toolbars(req)
+            chrome.add_auto_preview(req)
             data.update({'view': 'detail', 'enum': enum})
 
         else:
@@ -664,27 +637,12 @@ class AbstractEnumAdminPanel(TicketAdminPanel):
             if req.method == 'POST':
                 # Add enum
                 if req.args.get('add') and req.args.get('name'):
-                    name = req.args.get('name')
-                    try:
-                        enum = self._enum_cls(self.env, name=name)
-                    except ResourceNotFound:
-                        enum = self._enum_cls(self.env)
-                        enum.name = name
-                        try:
-                            enum.insert()
-                        except self.env.db_exc.IntegrityError:
-                            raise TracError(_('%(type)s value "%(name)s" '
-                                              'already exists', type=label[0],
-                                              name=enum.name))
-                        add_notice(req, _('The %(field)s value "%(name)s" '
-                                          'has been added.',
-                                          field=label[0], name=enum.name))
-                    else:
-                        if enum.name is None:
-                            raise TracError(_("Invalid %(type)s value.",
-                                              type=label[0]))
-                        raise TracError(_('%(type)s value "%(name)s" already '
-                                          'exists', type=label[0], name=name))
+                    enum = self._enum_cls(self.env)
+                    enum.name = req.args.get('name')
+                    enum.insert()
+                    add_notice(req, _('The %(field)s value "%(name)s" '
+                                      'has been added.',
+                                      field=label[0], name=enum.name))
 
                 # Remove enums
                 elif req.args.get('remove'):
@@ -713,11 +671,10 @@ class AbstractEnumAdminPanel(TicketAdminPanel):
                         self._save_config(req)
 
                     # Change enum values
-                    order = dict([(str(int(key[6:])),
-                                   str(int(req.args.get(key)))) for key
-                                  in req.args.keys()
-                                  if key.startswith('value_')])
-                    values = dict([(val, True) for val in order.values()])
+                    order = {str(int(key[6:])): str(req.args.getint(key))
+                             for key in req.args
+                             if key.startswith('value_')}
+                    values = {val: True for val in order.values()}
                     if len(order) != len(values):
                         raise TracError(_("Order numbers must be unique"))
                     changed = False
@@ -737,15 +694,14 @@ class AbstractEnumAdminPanel(TicketAdminPanel):
                     self.log.info("Clearing default %s", self._type)
                     self.config.set('ticket', 'default_%s' % self._type, '')
                     self._save_config(req)
-                    req.redirect(req.href.admin(cat, page))
 
                 req.redirect(req.href.admin(cat, page))
 
+            Chrome(self.env).add_jquery_ui(req)
+            add_script(req, 'common/js/admin_enums.js')
             data.update(dict(enums=list(self._enum_cls.select(self.env)),
                              default=default, view='list'))
 
-        Chrome(self.env).add_jquery_ui(req)
-        add_script(req, 'common/js/admin_enums.js')
         return 'admin_enums.html', data
 
     # IAdminCommandProvider methods
@@ -828,25 +784,25 @@ class AbstractEnumAdminPanel(TicketAdminPanel):
 class PriorityAdminPanel(AbstractEnumAdminPanel):
     _type = 'priority'
     _enum_cls = model.Priority
-    _label = N_("Priority"), N_("Priorities")
+    _label = model.Priority.label
 
 
 class ResolutionAdminPanel(AbstractEnumAdminPanel):
     _type = 'resolution'
     _enum_cls = model.Resolution
-    _label = N_("Resolution"), N_("Resolutions")
+    _label = model.Resolution.label
 
 
 class SeverityAdminPanel(AbstractEnumAdminPanel):
     _type = 'severity'
     _enum_cls = model.Severity
-    _label = N_("Severity"), N_("Severities")
+    _label = model.Severity.label
 
 
 class TicketTypeAdminPanel(AbstractEnumAdminPanel):
     _type = 'type'
     _enum_cls = model.Type
-    _label = N_("Ticket Type"), N_("Ticket Types")
+    _label = model.Type.label
 
     _command_type = 'ticket_type'
     _command_help = {
@@ -866,15 +822,33 @@ class TicketAdmin(Component):
     # IAdminCommandProvider methods
 
     def get_admin_commands(self):
-        yield ('ticket remove', '<number>', 'Remove ticket',
-               None, self._do_remove)
+        yield ('ticket remove', '<ticket#>',
+               'Remove ticket', None, self._do_remove)
+        yield ('ticket remove_comment', '<ticket#> <comment#>',
+               'Remove ticket comment', None, self._do_remove_comment)
 
     def _do_remove(self, number):
-        try:
-            number = int(number)
-        except ValueError:
-            raise AdminCommandError(_("<number> must be a number"))
+        number = as_int(number, None)
+        if number is None:
+            raise AdminCommandError(_("<ticket#> must be a number"))
         with self.env.db_transaction:
             model.Ticket(self.env, number).delete()
         printout(_("Ticket #%(num)s and all associated data removed.",
                    num=number))
+
+    def _do_remove_comment(self, ticket_number, comment_number):
+        ticket_number = as_int(ticket_number, None)
+        if ticket_number is None:
+            raise AdminCommandError(_('<ticket#> must be a number'))
+        comment_number = as_int(comment_number, None)
+        if comment_number is None:
+            raise AdminCommandError(_('<comment#> must be a number'))
+        with self.env.db_transaction:
+            ticket = model.Ticket(self.env, ticket_number)
+            change = ticket.get_change(comment_number)
+            if not change:
+                raise AdminCommandError(_("Comment %(num)s not found",
+                                          num=comment_number))
+            ticket.delete_change(comment_number)
+        printout(_("The ticket comment %(num)s on ticket #%(id)s has been "
+                   "deleted.", num=comment_number, id=ticket_number))
