@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2020 Edgewall Software
+# Copyright (C) 2004-2021 Edgewall Software
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -15,16 +15,14 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from cStringIO import StringIO
 from datetime import datetime, timedelta
 from itertools import groupby
 import operator
 from math import ceil
 import csv
+import io
 import re
 
-from genshi.builder import tag
-
 from trac.config import Option, IntOption
 from trac.core import *
 from trac.db import get_column_names
@@ -37,6 +35,7 @@ from trac.util import Ranges, as_bool, as_int
 from trac.util.datefmt import (datetime_now, from_utimestamp,
                                format_date_or_datetime, parse_date,
                                to_timestamp, to_utimestamp, utc, user_time)
+from trac.util.html import tag
 from trac.util.presentation import Paginator
 from trac.util.text import empty, shorten_line, quote_query_string
 from trac.util.translation import _, cleandoc_, ngettext, tag_
@@ -122,9 +121,9 @@ class Query(object):
         if verbose and 'description' not in rows:  # 0.10 compatibility
             rows.append('description')
         self.fields = TicketSystem(self.env).get_ticket_fields()
-        self.time_fields = set(f['name'] for f in self.fields
-                               if f['type'] == 'time')
-        field_names = set(f['name'] for f in self.fields)
+        self.time_fields = {f['name'] for f in self.fields
+                            if f['type'] == 'time'}
+        field_names = {f['name'] for f in self.fields}
         self.cols = [c for c in cols or [] if c in field_names or
                      c == 'id']
         self.rows = [c for c in rows if c in field_names]
@@ -240,8 +239,8 @@ class Query(object):
 
         # Semi-intelligently remove columns that are restricted to a single
         # value by a query constraint.
-        for col in [k for k in self.constraint_cols.keys()
-                    if k != 'id' and k in cols]:
+        for col in [k for k in self.constraint_cols
+                      if k != 'id' and k in cols]:
             constraints = self.constraint_cols[col]
             for constraint in constraints:
                 if not (len(constraint) == 1 and constraint[0]
@@ -332,7 +331,7 @@ class Query(object):
                         if href is not None:
                             result['href'] = href.ticket(val)
                     elif name in self.time_fields:
-                        val = from_utimestamp(long(val)) if val else None
+                        val = from_utimestamp(int(val)) if val else None
                     elif field and field['type'] == 'checkbox':
                         val = as_bool(val)
                     elif val is None:
@@ -439,10 +438,10 @@ class Query(object):
         add_cols('status', 'priority', 'time', 'changetime', self.order)
         add_cols(*list(self.constraint_cols))
 
-        custom_fields = set(f['name'] for f in self.fields if f.get('custom'))
-        list_fields = set(f['name'] for f in self.fields
-                                    if f['type'] == 'text' and
-                                       f.get('format') == 'list')
+        custom_fields = {f['name'] for f in self.fields if f.get('custom')}
+        list_fields = {f['name'] for f in self.fields
+                                 if f['type'] == 'text' and
+                                    f.get('format') == 'list'}
         cols_custom = [k for k in cols if k in custom_fields]
         use_joins = len(cols_custom) <= 1
         enum_columns = [col for col in ('resolution', 'priority', 'severity',
@@ -792,6 +791,12 @@ class Query(object):
                     for (label, milestones) in groups]
             fields[name] = field
 
+        def by_label(name):
+            if 'label' in fields[name]:
+                return fields[name]['label'].lower()
+            return ''
+        field_names = sorted(fields, key=by_label)
+
         groups = {}
         groupsequence = []
         for ticket in tickets:
@@ -859,6 +864,7 @@ class Query(object):
                 'clauses': clauses,
                 'headers': headers,
                 'fields': fields,
+                'field_names': field_names,
                 'modes': self.get_modes(),
                 'tickets': tickets,
                 'groups': groupsequence or [(None, tickets)],
@@ -891,7 +897,7 @@ class QueryModule(Component):
 
     items_per_page = IntOption('query', 'items_per_page', 100,
         """Number of tickets displayed per page in ticket queries,
-        by default.
+        by default. Set to `0` to specify no limit.
         """)
 
     # IContentConverter methods
@@ -919,11 +925,8 @@ class QueryModule(Component):
         return 'tickets'
 
     def get_navigation_items(self, req):
-        from trac.ticket.report import ReportModule
         if 'TICKET_VIEW' in req.perm(self.realm) and \
-                (not self.env.is_component_enabled(ReportModule) or
-                 'REPORT_VIEW' not in req.perm(ReportModule.realm,
-                                               ReportModule.REPORT_LIST_ID)):
+                 'REPORT_VIEW' not in req.perm('report', -1):
             yield ('mainnav', 'tickets',
                    tag.a(_("View Tickets"), href=req.href.query()))
 
@@ -942,7 +945,7 @@ class QueryModule(Component):
         args = req.args
         if not constraints:
             # If no constraints are given in the URL, use the default ones.
-            if req.authname and req.authname != 'anonymous':
+            if req.is_authenticated:
                 qstring = self.default_query
                 user = req.authname
             else:
@@ -977,29 +980,19 @@ class QueryModule(Component):
                             del clause[field]
                             break
 
-        cols = args.get('col')
-        if isinstance(cols, basestring):
-            cols = [cols]
+        cols = args.getlist('col')
         # Since we don't show 'id' as an option to the user,
         # we need to re-insert it here.
         if cols and 'id' not in cols:
             cols.insert(0, 'id')
-        rows = args.get('row', [])
-        if isinstance(rows, basestring):
-            rows = [rows]
+        rows = args.getlist('row')
         format = req.args.get('format')
         max = args.get('max')
         if max is None and format in ('csv', 'tab'):
             max = 0  # unlimited unless specified explicitly
         order = args.get('order')
-        if isinstance(order, (list, tuple)):
-            order = order[0] if order else None
         group = args.get('group')
-        if isinstance(group, (list, tuple)):
-            group = group[0] if group else None
         page = args.get('page')
-        if isinstance(page, (list, tuple)):
-            page = page[0] if page else None
         query = Query(self.env, report_id,
                       constraints, cols, order, as_bool(args.get('desc')),
                       group, as_bool(args.get('groupdesc')),
@@ -1037,7 +1030,7 @@ class QueryModule(Component):
     def _get_constraints(self, req=None, arg_list=[]):
         fields = TicketSystem(self.env).get_ticket_fields()
         synonyms = TicketSystem(self.env).get_field_synonyms()
-        fields = dict((f['name'], f) for f in fields)
+        fields = {f['name']: f for f in fields}
         fields['id'] = {'type': 'id'}
         fields.update((k, fields[v]) for k, v in synonyms.iteritems())
 
@@ -1155,10 +1148,8 @@ class QueryModule(Component):
         #
         # Note that with saved custom queries, there will be some convergence
         # between the report module and the query module.
-        from trac.ticket.report import ReportModule
-        report_resource = Resource(ReportModule.realm, query.id)
-        if 'REPORT_VIEW' in req.perm(report_resource) and \
-                self.env.is_component_enabled(ReportModule):
+        report_resource = Resource('report', query.id)
+        if 'REPORT_VIEW' in req.perm(report_resource):
             data['report_href'] = req.href.report()
             add_ctxtnav(req, _("Available Reports"), req.href.report())
             add_ctxtnav(req, _("New Custom Query"), req.href.query())
@@ -1181,32 +1172,22 @@ class QueryModule(Component):
         data['all_columns'].remove('id')
         data['all_textareas'] = query.get_all_textareas()
 
-        properties = dict((name, dict((key, field[key])
-                                      for key in ('type', 'label', 'options',
-                                                  'optgroups', 'optional',
-                                                  'format')
-                                      if key in field))
-                          for name, field in data['fields'].iteritems())
+        properties = {name: {key: field[key]
+                             for key in ('type', 'label', 'options',
+                                         'optgroups', 'optional', 'format')
+                             if key in field}
+                      for name, field in data['fields'].iteritems()}
         add_script_data(req, properties=properties, modes=data['modes'])
 
         add_stylesheet(req, 'common/css/report.css')
         Chrome(self.env).add_jquery_ui(req)
         add_script(req, 'common/js/query.js')
 
-        return 'query.html', data, None
-
-    def export_csv(self, req, query, sep=',', mimetype='text/plain'):
-        """Export as CSV
-
-        :deprecated: since 1.0.6, use `_export_csv` instead. Will be
-            removed in 1.3.1.
-        """
-        content, content_type = self._export_csv(req, query, sep, mimetype)
-        return ''.join(content), content_type
+        return 'query.html', data
 
     def _export_csv(self, req, query, sep=',', mimetype='text/plain'):
         def iterate():
-            out = StringIO()
+            out = io.BytesIO()
             writer = csv.writer(out, delimiter=sep, quoting=csv.QUOTE_MINIMAL)
 
             def writerow(values):
@@ -1214,6 +1195,7 @@ class QueryModule(Component):
                                  for value in values])
                 rv = out.getvalue()
                 out.truncate(0)
+                out.seek(0)
                 return rv
 
             yield '\xef\xbb\xbf'  # BOM
@@ -1248,15 +1230,6 @@ class QueryModule(Component):
 
         return iterate(), '%s;charset=utf-8' % mimetype
 
-    def export_rss(self, req, query):
-        """Export as RSS
-
-        :deprecated: since 1.0.6, use `_export_rss` instead. Will be
-            removed in 1.3.1.
-        """
-        content, content_type = self._export_rss(req, query)
-        return ''.join(content), content_type
-
     def _export_rss(self, req, query):
         context = web_context(req, 'query', absurls=True)
         query_href = query.get_href(context.href)
@@ -1269,8 +1242,9 @@ class QueryModule(Component):
             'query_href': query_href
         }
         output = Chrome(self.env).render_template(req, 'query.rss', data,
-                                                  'application/rss+xml',
-                                                  iterable=True)
+                                                  {'content_type':
+                                                   'application/rss+xml',
+                                                   'iterable': True})
         return output, 'application/rss+xml'
 
     # IWikiSyntaxProvider methods
@@ -1440,8 +1414,8 @@ class TicketQueryMacro(WikiMacroBase):
 
             add_stylesheet(req, 'common/css/report.css')
 
-            return Chrome(self.env).render_template(
-                req, 'query_results.html', data, None, fragment=True)
+            return Chrome(self.env).render_fragment(req, 'query_results.html',
+                                                    data)
 
         if format == 'progress':
             from trac.ticket.roadmap import (RoadmapModule,
@@ -1483,8 +1457,7 @@ class TicketQueryMacro(WikiMacroBase):
                     'legend': True,
                 }
                 return tag.div(
-                    chrome.render_template(req, 'progress_bar.html', data,
-                                           None, fragment=True),
+                    chrome.render_fragment(req, 'progress_bar.html', data),
                     class_='trac-progress')
 
             def per_group_stats_data(gstat, group_name):
@@ -1509,8 +1482,7 @@ class TicketQueryMacro(WikiMacroBase):
                              group=by),
             }
             return tag.div(
-                chrome.render_template(req, 'progress_bar_grouped.html', data,
-                                       None, fragment=True),
+                chrome.render_fragment(req, 'progress_bar_grouped.html', data),
                 class_='trac-groupprogress')
 
         # Formats above had their own permission checks, here we need to
