@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2020 Edgewall Software
+# Copyright (C) 2004-2021 Edgewall Software
 # Copyright (C) 2004 Francois Harvey <fharvey@securiweb.net>
 # Copyright (C) 2005 Matthew Good <trac@matt-good.net>
 # All rights reserved.
@@ -18,12 +18,12 @@
 
 import os.path
 
-from trac.config import ConfigurationError, Option, PathOption
-from trac.core import *
+from trac.config import ConfigurationError, Option, ParsingError, \
+                        PathOption, UnicodeConfigParser
+from trac.core import Component, TracError, implements
 from trac.perm import IPermissionPolicy
-from trac.util import read_file
-from trac.util.text import exception_to_unicode, to_unicode
-from trac.util.translation import _
+from trac.util import pathjoin, to_list
+from trac.util.text import exception_to_unicode
 from trac.versioncontrol.api import RepositoryManager
 
 
@@ -38,54 +38,33 @@ def parent_iter(path):
         path = path[:idx + 1]
 
 
-def join(*args):
-    args = (arg.strip('/') for arg in args)
-    return '/'.join(arg for arg in args if arg)
-
-
-class ParseError(TracBaseError):
-    """Exception thrown for parse errors in authz files"""
-
-
-def parse(authz, modules):
+def parse(authz_file, modules):
     """Parse a Subversion authorization file.
 
     Return a dict of modules, each containing a dict of paths, each containing
     a dict mapping users to permissions. Only modules contained in `modules`
     are retained.
     """
+    parser = UnicodeConfigParser(ignorecase_option=False)
+    parser.read(authz_file)
+
     groups = {}
     aliases = {}
     sections = {}
-    section = None
-    lineno = 0
-    for line in authz.splitlines():
-        lineno += 1
-        line = to_unicode(line.strip())
-        if not line or line.startswith(('#', ';')):
-            continue
-        if line.startswith('[') and line.endswith(']'):
-            section = line[1:-1]
-            continue
-        if section is None:
-            raise ParseError(_('Line %(lineno)d: Entry before first '
-                               'section header', lineno=lineno))
-        parts = line.split('=', 1)
-        if len(parts) != 2:
-            raise ParseError(_('Line %(lineno)d: Invalid entry',
-                               lineno=lineno))
-        name, value = parts
-        name = name.strip()
+    for section in parser.sections():
         if section == 'groups':
-            group = groups.setdefault(name, set())
-            group.update(each.strip() for each in value.split(','))
+            for name, value in parser.items(section):
+                groups.setdefault(name, set()).update(to_list(value))
         elif section == 'aliases':
-            aliases[name] = value.strip()
+            for name, value in parser.items(section):
+                aliases[name] = value.strip()
         else:
-            parts = section.split(':', 1)
-            module, path = parts[0] if len(parts) > 1 else '', parts[-1]
-            if module in modules:
-                sections.setdefault((module, path), []).append((name, value))
+            for name, value in parser.items(section):
+                parts = section.split(':', 1)
+                module, path = parts[0] if len(parts) > 1 else '', parts[-1]
+                if module in modules:
+                    sections.setdefault((module, path), []) \
+                            .append((name, value))
 
     def resolve(subject, done):
         if subject.startswith('@'):
@@ -126,11 +105,13 @@ class AuthzSourcePolicy(Component):
     authz_file = PathOption('svn', 'authz_file', '',
         """The path to the Subversion
         [%(svnbook)s authorization (authz) file].
-        To enable authz permission checking, the `AuthzSourcePolicy` permission
-        policy must be added to `[trac] permission_policies`. Non-absolute
-        paths are relative to the Environment `conf` directory.
+        To enable authz permission checking, the `AuthzSourcePolicy`
+        permission policy must be added to `[trac] permission_policies`.
+        Non-absolute paths are relative to the Environment `conf`
+        directory.
         """,
-        doc_args={'svnbook': 'http://svnbook.red-bean.com/en/1.7/svn.serverconfig.pathbasedauthz.html'})
+        doc_args={'svnbook': 'http://svnbook.red-bean.com/en/1.7/'
+                             'svn.serverconfig.pathbasedauthz.html'})
 
     authz_module_name = Option('svn', 'authz_module_name', '',
         """The module prefix used in the `authz_file` for the default
@@ -161,9 +142,9 @@ class AuthzSourcePolicy(Component):
                 return False
 
             if username == 'anonymous':
-                usernames = ('$anonymous', '*')
+                usernames = '$anonymous', '*'
             else:
-                usernames = (username, '$authenticated', '*')
+                usernames = username, '$authenticated', '*'
             if resource is None:
                 return True if users & set(usernames) else None
 
@@ -171,7 +152,7 @@ class AuthzSourcePolicy(Component):
             try:
                 repos = rm.get_repository(resource.parent.id)
             except TracError:
-                return True # Allow error to be displayed in the repo index
+                return True  # Allow error to be displayed in the repo index
             if repos is None:
                 return True
             modules = [resource.parent.id or self.authz_module_name]
@@ -197,12 +178,12 @@ class AuthzSourcePolicy(Component):
                     return False
 
             def check_path(path):
-                path = '/' + join(repos.scope, path)
+                path = '/' + pathjoin(repos.scope, path)
                 if path != '/':
                     path += '/'
 
                 # Allow access to parent directories of allowed resources
-                for spath in set(sum((authz.get(module, {}).keys()
+                for spath in set(sum((list(authz.get(module, {}))
                                       for module in modules), [])):
                     if spath.startswith(path):
                         result = check_path_0(spath)
@@ -243,16 +224,17 @@ class AuthzSourcePolicy(Component):
             if '' in modules and self.authz_module_name:
                 modules.add(self.authz_module_name)
             modules.add('')
-            self.log.info('Parsing authz file: %s', self.authz_file)
+            self.log.info("Parsing authz file: %s", self.authz_file)
             try:
-                self._authz = parse(read_file(self.authz_file), modules)
-            except Exception as e:
+                self._authz = parse(self.authz_file, modules)
+            except ParsingError as e:
                 self.log.error("Error parsing svn authz permission policy "
                                "file: %s", exception_to_unicode(e))
                 raise ConfigurationError()
             else:
-                self._users = set(user for paths in self._authz.itervalues()
-                                  for path in paths.itervalues()
-                                  for user, result in path.iteritems()
-                                  if result)
+                self._users = {user
+                               for paths in self._authz.itervalues()
+                               for path in paths.itervalues()
+                               for user, result in path.iteritems()
+                               if result}
         return self._authz, self._users
