@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
@@ -24,8 +24,6 @@ import os
 import posixpath
 import re
 
-from genshi.builder import tag
-
 from trac.config import BoolOption, IntOption, Option
 from trac.core import *
 from trac.mimeview.api import Mimeview
@@ -35,6 +33,7 @@ from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import as_bool, content_disposition, embedded_numbers, pathjoin
 from trac.util.datefmt import from_utimestamp, pretty_timedelta
+from trac.util.html import tag
 from trac.util.presentation import to_json
 from trac.util.text import CRLF, exception_to_unicode, shorten_line, \
                            to_unicode, unicode_urlencode
@@ -78,8 +77,7 @@ class IPropertyDiffRenderer(Interface):
          - `None`: the property change will be shown the normal way
            (''changed from `old` to `new`'')
          - an `unicode` value: the change will be shown as textual content
-         - `Markup` or other Genshi content: the change will shown as block
-           markup
+         - `Markup` or `Fragment`: the change will shown as block markup
         """
 
 
@@ -374,7 +372,7 @@ class ChangesetModule(Component):
                                           old=new, old_path=full_new_path)
             add_ctxtnav(req, _('Reverse Diff'), href=rev_href)
 
-        return 'changeset.html', data, None
+        return 'changeset.html', data
 
     # Internal methods
 
@@ -561,8 +559,7 @@ class ChangesetModule(Component):
                 context = options.get('contextlines', 3)
                 if context < 0 or options.get('contextall'):
                     context = None
-                tabwidth = self.config['diff'].getint('tab_width') or \
-                           self.config['mimeviewer'].getint('tab_width', 8)
+                tabwidth = self.config.getint('mimeviewer', 'tab_width', 8)
                 ignore_blank_lines = options.get('ignoreblanklines')
                 ignore_case = options.get('ignorecase')
                 ignore_space = options.get('ignorewhitespace')
@@ -666,7 +663,6 @@ class ChangesetModule(Component):
             'max_diff_files': self.max_diff_files,
             'max_diff_bytes': self.max_diff_bytes,
             'changes': changes,
-            'xhr': req.is_xhr,  # Remove in 1.3.1
             'filestats': filestats,
             'annotated': annotated,
             'files': files,
@@ -676,15 +672,9 @@ class ChangesetModule(Component):
         })
 
         if req.is_xhr:  # render and return the content only
-            chrome = Chrome(self.env)
-            stream = chrome.render_template(req, 'changeset.html', data,
-                                            fragment=True)
-            stream = stream.select('//div[@id="content"]')
-            if chrome.use_chunked_encoding:
-                output = chrome.iterable_content(stream, 'xhtml')
-            else:
-                output = stream.render('xhtml', encoding='utf-8')
-            req.send(output)
+            stream = Chrome(self.env).generate_fragment(
+                req, 'changeset_content.html', data)
+            req.send(stream)
 
         return data
 
@@ -878,7 +868,7 @@ class ChangesetModule(Component):
 
     def get_timeline_events(self, req, start, stop, filters):
         all_repos = 'changeset' in filters
-        repo_filters = set(f for f in filters if f.startswith('repo-'))
+        repo_filters = {f for f in filters if f.startswith('repo-')}
         if all_repos or repo_filters:
             show_files = self.timeline_show_files
             show_location = show_files == 'location'
@@ -1030,8 +1020,8 @@ class ChangesetModule(Component):
                 labels.append(tag.span(name, class_='trac-bookmark'))
             return title if not labels else tag(title, labels)
         elif field == 'summary':
-            return _("%(title)s: %(message)s",
-                     title=title, message=shorten_line(message))
+            return tag_("%(title)s: %(message)s",
+                        title=title, message=shorten_line(message))
 
     # IWikiSyntaxProvider methods
 
@@ -1142,8 +1132,8 @@ class ChangesetModule(Component):
         if not 'changeset' in filters:
             return
         rm = RepositoryManager(self.env)
-        repositories = dict((repos.params['id'], repos)
-                            for repos in rm.get_real_repositories())
+        repositories = {repos.params['id']: repos
+                        for repos in rm.get_real_repositories()}
         uids_seen = set()
         with self.env.db_query as db:
             sql, args = search_to_sql(db, ['rev', 'message', 'author'], terms)
@@ -1188,7 +1178,7 @@ class AnyDiffModule(Component):
             reponame, repos, path = rm.get_repository_by_path(dirname)
             # an entry is a (isdir, name, path) tuple
             def kind_order(entry):
-                return (not entry[0], embedded_numbers(entry[1]))
+                return not entry[0], embedded_numbers(entry[1])
 
             entries = []
             if repos:
@@ -1235,7 +1225,7 @@ class AnyDiffModule(Component):
 
         Chrome(self.env).add_jquery_ui(req)
         add_stylesheet(req, 'common/css/diff.css')
-        return 'diff_form.html', data, None
+        return 'diff_form.html', data
 
 
 def _read_content(node):
