@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -66,12 +66,14 @@ class _FileWrapper(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         data = self.fileobj.read(self.blocksize)
         if not data:
             raise StopIteration
         return data
 
+    next = __next__
+
 
 class WSGIGateway(object):
     """Abstract base class for WSGI servers or gateways."""
