@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
@@ -18,13 +18,11 @@
 
 import re
 
-from genshi.builder import tag
-
 from trac.cache import cached
 from trac.config import BoolOption, ListOption
 from trac.core import *
 from trac.resource import IResourceManager
-from trac.util.html import is_safe_origin
+from trac.util.html import is_safe_origin, tag
 from trac.util.text import unquote_label
 from trac.util.translation import _
 from trac.wiki.parser import WikiParser
@@ -39,12 +37,8 @@ class IWikiChangeListener(Interface):
     def wiki_page_added(page):
         """Called whenever a new Wiki page is added."""
 
-    def wiki_page_changed(page, version, t, comment, author, ipnr):
-        """Called when a page has been modified.
-
-        :since 1.0.3: `ipnr` is optional and deprecated, and will
-                      be removed in 1.3.1
-        """
+    def wiki_page_changed(page, version, t, comment, author):
+        """Called when a page has been modified."""
 
     def wiki_page_deleted(page):
         """Called when a page has been deleted."""
@@ -109,6 +103,10 @@ class IWikiMacroProvider(Interface):
         .. versionchanged :: 1.0
            `get_macro_description` can return a domain to translate the
            description.
+
+        .. versionchanged :: 1.3.6
+           the macro will be hidden from the macro index (`[[MacroList]]`)
+           if `None` is returned.
         """
 
     def is_inline(content):
@@ -254,13 +252,15 @@ def validate_page_name(pagename):
 class WikiSystem(Component):
     """Wiki system manager."""
 
-    implements(IWikiSyntaxProvider, IResourceManager)
+    implements(IResourceManager, IWikiSyntaxProvider)
 
     change_listeners = ExtensionPoint(IWikiChangeListener)
     macro_providers = ExtensionPoint(IWikiMacroProvider)
     syntax_providers = ExtensionPoint(IWikiSyntaxProvider)
 
     realm = 'wiki'
+    START_PAGE = 'WikiStart'
+    TITLE_INDEX_PAGE = 'TitleIndex'
 
     ignore_missing_pages = BoolOption('wiki', 'ignore_missing_pages', 'false',
         """Enable/disable highlighting CamelCase links to missing pages.
@@ -297,8 +297,8 @@ class WikiSystem(Component):
     @cached
     def pages(self):
         """Return the names of all existing wiki pages."""
-        return set(name for name, in
-                   self.env.db_query("SELECT DISTINCT name FROM wiki"))
+        return {name for name,
+                     in self.env.db_query("SELECT DISTINCT name FROM wiki")}
 
     # Public API
 
@@ -333,8 +333,8 @@ class WikiSystem(Component):
 
     PAGE_SPLIT_RE = re.compile(r"([a-z])([A-Z])(?=[a-z])")
 
-    Lu = ''.join(unichr(c) for c in range(0, 0x10000) if unichr(c).isupper())
-    Ll = ''.join(unichr(c) for c in range(0, 0x10000) if unichr(c).islower())
+    Lu = ''.join(unichr(c) for c in xrange(0x10000) if unichr(c).isupper())
+    Ll = ''.join(unichr(c) for c in xrange(0x10000) if unichr(c).islower())
 
     def format_page_name(self, page, split=False):
         if split or self.split_page_names:
@@ -420,7 +420,7 @@ class WikiSystem(Component):
             pagename, version = pagename.split('@', 1)
         if version and query:
             query = '&' + query[1:]
-        pagename = pagename.rstrip('/') or 'WikiStart'
+        pagename = pagename.rstrip('/') or self.START_PAGE
         referrer = ''
         if formatter.resource and formatter.resource.realm == self.realm:
             referrer = formatter.resource.id
@@ -441,10 +441,10 @@ class WikiSystem(Component):
                     return original_label or label
                 if 'WIKI_CREATE' in \
                         formatter.perm(self.realm, pagename, version):
-                    return tag.a(label + '?', class_='missing wiki',
+                    return tag.a(label, class_='missing wiki',
                                  href=href, rel='nofollow')
                 else:
-                    return tag.a(label + '?', class_='missing wiki')
+                    return tag.a(label, class_='missing wiki')
         elif ignore_missing and not self.has_page(pagename):
             return original_label or label
         else:
@@ -468,7 +468,7 @@ class WikiSystem(Component):
         if len(referrer) == 1:           # Non-hierarchical referrer
             return pagename
         # Test for pages with same name, higher in the hierarchy
-        for i in range(len(referrer) - 1, 0, -1):
+        for i in xrange(len(referrer) - 1, 0, -1):
             name = '/'.join(referrer[:i]) + '/' + pagename
             if self.has_page(name):
                 return name
@@ -525,7 +525,7 @@ class WikiSystem(Component):
         >>> from trac.wiki.model import WikiPage
         >>> main = WikiPage(env, 'WikiStart')
         >>> main.text = 'some content'
-        >>> main.save('author', 'no comment', '::1')
+        >>> main.save('author', 'no comment')
         >>> resource_exists(env, main.resource)
         True
         """
