@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
@@ -18,33 +18,40 @@
 #         Christopher Lenz <cmlenz@gmx.de>
 #         Christian Boos <cboos@edgewall.org>
 
+from HTMLParser import HTMLParseError
+import io
 import re
-import os
-
-from StringIO import StringIO
-
-from genshi.builder import tag, Element
-from genshi.core import Stream
-from genshi.input import HTMLParser, ParseError
-from genshi.util import plaintext
 
 from trac.core import *
 from trac.mimeview import *
 from trac.resource import get_relative_resource, get_resource_url
 from trac.util import arity, as_int
-from trac.util.html import Markup, TracHTMLSanitizer, escape, to_fragment
-from trac.util.text import exception_to_unicode, shorten_line, to_unicode, \
-                           unicode_quote, unquote_label
+from trac.util.text import (
+    exception_to_unicode, shorten_line, to_unicode, unicode_quote,
+    unquote_label
+)
+from trac.util.html import (
+    Element, Fragment, Markup, Stream, TracHTMLSanitizer, escape, genshi,
+    plaintext, stream_to_unicode, tag, to_fragment
+)
 from trac.util.translation import _, tag_
 from trac.wiki.api import WikiSystem, parse_args
 from trac.wiki.parser import WikiParser, parse_processor_args
 
-__all__ = ['wiki_to_html', 'wiki_to_oneliner', 'wiki_to_outline',
-           'Formatter', 'MacroError', 'ProcessorError', 'format_to',
-           'format_to_html', 'format_to_oneliner', 'extract_link',
-           'split_url_into_path_query_fragment', 'concat_path_query_fragment']
+__all__ = ['Formatter', 'MacroError', 'ProcessorError',
+           'concat_path_query_fragment', 'extract_link', 'format_to',
+           'format_to_html', 'format_to_oneliner',
+           'split_url_into_path_query_fragment', 'wiki_to_outline']
 
 
+def _markup_to_unicode(markup):
+    if isinstance(markup, Fragment):
+        return Markup(markup)
+    elif genshi and isinstance(markup, Stream):
+        return stream_to_unicode(markup)
+    else:
+        return to_unicode(markup)
+
 
 def system_message(msg, text=None):
     return tag.div(tag.strong(msg), text and tag.pre(text),
@@ -100,17 +107,6 @@ def concat_path_query_fragment(path, query, fragment=None):
     return p + q + ('' if f == '#' else f)
 
 
-def _markup_to_unicode(markup):
-    stream = None
-    if isinstance(markup, Element):
-        stream = markup.generate()
-    elif isinstance(markup, Stream):
-        stream = markup
-    if stream:
-        markup = stream.render('xhtml', encoding=None, strip_whitespace=False)
-    return to_unicode(markup)
-
-
 class MacroError(TracError):
     """Exception raised on incorrect macro usage.
 
@@ -121,7 +117,6 @@ class MacroError(TracError):
     """
     pass
 
-
 class ProcessorError(TracError):
     """Exception raised on incorrect processor usage.
 
@@ -244,10 +239,9 @@ class WikiProcessor(object):
         if WikiSystem(self.env).render_unsafe_content:
             return Markup(text)
         try:
-            stream = Stream(HTMLParser(StringIO(text)))
-            return (stream | self._sanitizer).render('xhtml', encoding=None)
-        except ParseError as e:
-            self.env.log.warn(e)
+            return self._sanitizer.sanitize(text)
+        except HTMLParseError as e:
+            self.env.log.warning(e)
             line = unicode(text).splitlines()[e.lineno - 1].strip()
             return system_message(_('HTML parsing error: %(message)s',
                                     message=escape(e.msg)), line)
@@ -256,7 +250,7 @@ class WikiProcessor(object):
         if "--" in text:
             return system_message(_('Error: Forbidden character sequence '
                                     '"--" in htmlcomment wiki code block'))
-        return Markup('<!--\n%s-->\n' % text)
+        return Markup(u'<!--\n%s-->\n' % text)
 
     def _elt_processor(self, eltname, format_to, text):
         # Note: as long as _processor_param_re is not re.UNICODE, **args is OK.
@@ -264,9 +258,8 @@ class WikiProcessor(object):
         elt = getattr(tag, eltname)(**(self.args or {}))
         if not WikiSystem(self.env).render_unsafe_content:
             sanitized_elt = getattr(tag, eltname)
-            for (k, data, pos) in (Stream(elt) | self._sanitizer):
-                sanitized_elt.attrib = data[1]
-                break # only look at START (elt,attrs)
+            sanitized_elt.attrib = self._sanitizer.sanitize_attrs(eltname,
+                                                                  elt.attrib)
             elt = sanitized_elt
         elt.append(format_to(self.env, self.formatter.context, text))
         return elt
@@ -343,14 +336,14 @@ class WikiProcessor(object):
 
     def _format_row(self, env, context, text):
         if text:
-            out = StringIO()
+            out = io.StringIO()
             Formatter(env, context).format(text, out)
             text = self._parse_inner_table(out.getvalue())
         return text
 
     def _format_table(self, env, context, text):
         if text:
-            out = StringIO()
+            out = io.StringIO()
             Formatter(env, context).format(text, out)
             text = self._parse_inner_table(out.getvalue())
         return text
@@ -436,7 +429,7 @@ class WikiProcessor(object):
 class Formatter(object):
     """Base Wiki formatter.
 
-    Parses and formats wiki text, in a given `Context`.
+    Parses and formats wiki text, in a given `RenderingContext`.
     """
 
     flavor = 'default'
@@ -464,14 +457,14 @@ class Formatter(object):
     # -- Pre- IWikiSyntaxProvider rules (Font styles)
 
     _indirect_tags = {
-        'MM_BOLD': ('<strong>', '</strong>'),
-        'WC_BOLD': ('<strong>', '</strong>'),
-        'MM_ITALIC': ('<em>', '</em>'),
-        'WC_ITALIC': ('<em>', '</em>'),
-        'MM_UNDERLINE': ('<span class="underline">', '</span>'),
-        'MM_STRIKE': ('<del>', '</del>'),
-        'MM_SUBSCRIPT': ('<sub>', '</sub>'),
-        'MM_SUPERSCRIPT': ('<sup>', '</sup>'),
+        'MM_BOLD': (u'<strong>', u'</strong>'),
+        'WC_BOLD': (u'<strong>', u'</strong>'),
+        'MM_ITALIC': (u'<em>', u'</em>'),
+        'WC_ITALIC': (u'<em>', u'</em>'),
+        'MM_UNDERLINE': (u'<span class="underline">', u'</span>'),
+        'MM_STRIKE': (u'<del>', u'</del>'),
+        'MM_SUBSCRIPT': (u'<sub>', u'</sub>'),
+        'MM_SUPERSCRIPT': (u'<sup>', u'</sup>'),
     }
 
     def _get_open_tag(self, tag):
@@ -490,9 +483,13 @@ class Formatter(object):
         """Do we currently have any open tag with `tag` as end-tag?"""
         return tag in self._open_tags
 
+    def pop_tags(self):
+        while self._open_tags:
+            yield self._open_tags.pop()
+
     def flush_tags(self):
-        while self._open_tags != []:
-            self.out.write(self._get_close_tag(self._open_tags.pop()))
+        for tag in self.pop_tags():
+            self.out.write(self._get_close_tag(tag))
 
     def open_tag(self, tag_open, tag_close=None):
         """Open an inline style tag.
@@ -622,7 +619,8 @@ class Formatter(object):
     # HTML escape of &, < and >
 
     def _htmlescape_formatter(self, match, fullmatch):
-        return "&amp;" if match == "&" else "&lt;" if match == "<" else "&gt;"
+        return u"&amp;" if match == "&" \
+                        else u"&lt;" if match == "<" else u"&gt;"
 
     # Short form (shref) and long form (lhref) of TracLinks
 
@@ -811,12 +809,12 @@ class Formatter(object):
 
     def _macro_formatter(self, match, fullmatch, macro, only_inline=False):
         name = fullmatch.group('macroname')
-        if name.lower() == 'br':
-            return '<br />'
         if name and name[-1] == '?': # Macro?() shortcut for MacroList(Macro)
             args = name[:-1] or '*'
         else:
             args = fullmatch.group('macroargs')
+        if name.lower() == 'br':
+            return self.emit_linebreak(args)
         in_paragraph = not (getattr(self, 'in_list_item', True) or
                             getattr(self, 'in_table', True) or
                             getattr(self, 'in_def_list', True))
@@ -831,6 +829,14 @@ class Formatter(object):
                                exception_to_unicode(e, traceback=True))
             return system_message(_("Error: Macro %(name)s(%(args)s) failed",
                                     name=name, args=args), to_fragment(e))
+    def emit_linebreak(self, args):
+        if args:
+            sep = ':' if ':' in args else '='
+            kv = args.split(sep, 1)
+            if kv[0] == 'clear':
+                clear = kv[-1] if kv[-1] in ['left', 'right'] else 'both'
+                return '<br style="clear: {0}" />'.format(clear)
+        return '<br />'
 
     # Headings
 
@@ -855,7 +861,7 @@ class Formatter(object):
         anchor = self._unique_anchor(anchor)
         if shorten:
             heading = format_to_oneliner(self.env, self.context, htext, True)
-        return (depth, heading, anchor)
+        return depth, heading, anchor
 
     def _heading_formatter(self, match, fullmatch):
         self.close_table()
@@ -864,7 +870,7 @@ class Formatter(object):
         self.close_list()
         self.close_def_list()
         depth, heading, anchor = self._parse_heading(match, fullmatch, False)
-        self.out.write('<h%d id="%s">%s</h%d>' %
+        self.out.write(u'<h%d class="section" id="%s">%s</h%d>' %
                        (depth, anchor, heading, depth))
 
     # Generic indentation (as defined by lists and quotes)
@@ -928,14 +934,14 @@ class Formatter(object):
             self._set_tab(depth)
             class_attr = ' class="%s"' % lclass if lclass else ''
             start_attr = ' start="%s"' % start if start is not None else ''
-            self.out.write('<' + new_type + class_attr + start_attr + '><li>')
+            self.out.write(u'<' + new_type + class_attr + start_attr + '><li>')
         def close_item():
             self.flush_tags()
-            self.out.write('</li>')
+            self.out.write(u'</li>')
         def close_list(tp):
             self._list_stack.pop()
             close_item()
-            self.out.write('</%s>' % tp)
+            self.out.write(u'</%s>' % tp)
 
         # depending on the indent/dedent, open or close lists
         if depth > self._get_list_depth():
@@ -956,7 +962,7 @@ class Formatter(object):
                         if old_offset != depth: # adjust last depth
                             self._list_stack[-1] = (old_type, depth)
                         close_item()
-                        self.out.write('<li>')
+                        self.out.write(u'<li>')
                 else:
                     open_list()
 
@@ -979,7 +985,7 @@ class Formatter(object):
 
     def close_def_list(self):
         if self.in_def_list:
-            self.out.write('</dd></dl>\n')
+            self.out.write(u'</dd></dl>\n')
         self.in_def_list = False
 
     # Blockquote
@@ -1017,9 +1023,9 @@ class Formatter(object):
                 self._quote_stack.append(d)
                 self._set_tab(d)
                 class_attr = ' class="citation"' if citation else ''
-                self.out.write('<blockquote%s>\n' % class_attr)
+                self.out.write(u'<blockquote%s>\n' % class_attr)
             if citation:
-                for d in range(quote_depth+1, depth+1):
+                for d in xrange(quote_depth+1, depth+1):
                     open_one_quote(d)
             else:
                 open_one_quote(depth)
@@ -1027,7 +1033,7 @@ class Formatter(object):
             self.close_table()
             self.close_paragraph()
             self._quote_stack.pop()
-            self.out.write('</blockquote>\n')
+            self.out.write(u'</blockquote>\n')
         quote_depth = self._get_quote_depth()
         if depth > quote_depth:
             self._set_tab(depth)
@@ -1099,7 +1105,9 @@ class Formatter(object):
             attrs += ' style="text-align: %s"' % textalign
         td = '<%s%s>' % (cell, attrs)
         if self.in_table_cell:
-            td = '</%s>' % self.in_table_cell + td
+            close_tags = ''.join(self._get_close_tag(tag)
+                                 for tag in self.pop_tags())
+            td = '%s</%s>%s' % (close_tags, self.in_table_cell, td)
         self.in_table_cell = cell
         return td
 
@@ -1121,40 +1129,41 @@ class Formatter(object):
             self.close_list()
             self.close_def_list()
             self.in_table = 1
-            self.out.write('<table class="wiki">\n')
+            self.out.write(u'<table class="wiki">\n')
 
     def open_table_row(self, params=''):
         if not self.in_table_row:
             self.open_table()
             self.in_table_row = 1
-            self.out.write('<tr%s>' % params)
+            self.out.write(u'<tr%s>' % params)
 
     def close_table_row(self, force=False):
         if self.in_table_row and (not self.continue_table_row or force):
             self.in_table_row = 0
             if self.in_table_cell:
-                self.out.write('</%s>' % self.in_table_cell)
+                self.flush_tags()
+                self.out.write(u'</%s>' % self.in_table_cell)
                 self.in_table_cell = ''
-            self.out.write('</tr>')
+            self.out.write(u'</tr>')
         self.continue_table_row = 0
 
     def close_table(self):
         if self.in_table:
             self.close_table_row(force=True)
-            self.out.write('</table>\n')
+            self.out.write(u'</table>\n')
             self.in_table = 0
 
     # Paragraphs
 
     def open_paragraph(self):
         if not self.paragraph_open:
-            self.out.write('<p>\n')
+            self.out.write(u'<p>\n')
             self.paragraph_open = 1
 
     def close_paragraph(self):
         self.flush_tags()
         if self.paragraph_open:
-            self.out.write('</p>\n')
+            self.out.write(u'</p>\n')
             self.paragraph_open = 0
 
     # Code blocks
@@ -1251,10 +1260,10 @@ class Formatter(object):
             if all(not line or line[0] in '> ' for line in self._quote_buffer):
                 self._quote_buffer = [line[bool(line and line[0] == ' '):]
                                       for line in self._quote_buffer]
-            self.out.write('<blockquote class="citation">\n')
+            self.out.write(u'<blockquote class="citation">\n')
             Formatter(self.env, self.context).format(self._quote_buffer,
                                                      self.out, escape_newlines)
-            self.out.write('</blockquote>\n')
+            self.out.write(u'</blockquote>\n')
             self._quote_buffer = []
 
     # -- Wiki engine
@@ -1310,6 +1319,8 @@ class Formatter(object):
             text = text.splitlines()
 
         for line in text:
+            if isinstance(line, str):
+                line = line.decode('utf-8')
             # Detect start of code block (new block or embedded block)
             block_start_match = None
             if WikiParser.ENDBLOCK not in line:
@@ -1335,7 +1346,7 @@ class Formatter(object):
                 self.close_indentation()
                 self.close_list()
                 self.close_def_list()
-                self.out.write('<hr />\n')
+                self.out.write(u'<hr />\n')
                 continue
             # Handle new paragraph
             if line == '':
@@ -1438,8 +1449,10 @@ class OneLinerFormatter(Formatter):
         # Simplify code blocks
         in_code_block = 0
         processor = None
-        buf = StringIO()
+        buf = io.StringIO()
         for line in text.strip().splitlines():
+            if isinstance(line, str):
+                line = line.decode('utf-8')
             if WikiParser.ENDBLOCK not in line and \
                    WikiParser._startblock_re.match(line):
                 in_code_block += 1
@@ -1448,7 +1461,7 @@ class OneLinerFormatter(Formatter):
                     in_code_block -= 1
                     if in_code_block == 0:
                         if processor != 'comment':
-                            buf.write(' [...]\n')
+                            buf.write(u' [...]\n')
                         processor = None
             elif in_code_block:
                 if not processor:
@@ -1507,30 +1520,30 @@ class OutlineFormatter(Formatter):
         min_depth = max(1, min_depth)
 
         curr_depth = min_depth - 1
-        out.write('\n')
+        out.write(u'\n')
         for depth, anchor, text in self.outline:
             if depth < min_depth or depth > max_depth:
                 continue
             if depth > curr_depth: # Deeper indent
-                for i in range(curr_depth, depth):
-                    out.write(whitespace_indent * (2*i) + '<ol>\n' +
-                              whitespace_indent * (2*i+1) + '<li>\n')
+                for i in xrange(curr_depth, depth):
+                    out.write(whitespace_indent * (2*i) + u'<ol>\n' +
+                              whitespace_indent * (2*i+1) + u'<li>\n')
             elif depth < curr_depth: # Shallower indent
-                for i in range(curr_depth-1, depth-1, -1):
-                    out.write(whitespace_indent * (2*i+1) + '</li>\n' +
-                              whitespace_indent * (2*i) + '</ol>\n')
-                out.write(whitespace_indent * (2*depth-1) + '</li>\n' +
-                          whitespace_indent * (2*depth-1) + '<li>\n')
+                for i in xrange(curr_depth-1, depth-1, -1):
+                    out.write(whitespace_indent * (2*i+1) + u'</li>\n' +
+                              whitespace_indent * (2*i) + u'</ol>\n')
+                out.write(whitespace_indent * (2*depth-1) + u'</li>\n' +
+                          whitespace_indent * (2*depth-1) + u'<li>\n')
             else: # Same indent
-                out.write( whitespace_indent * (2*depth-1) + '</li>\n' +
-                           whitespace_indent * (2*depth-1) + '<li>\n')
+                out.write( whitespace_indent * (2*depth-1) + u'</li>\n' +
+                           whitespace_indent * (2*depth-1) + u'<li>\n')
             curr_depth = depth
             out.write(whitespace_indent * (2*depth) +
-                      '<a href="#%s">%s</a>\n' % (anchor, text))
+                      u'<a href="#%s">%s</a>\n' % (anchor, text))
         # Close out all indentation
-        for i in range(curr_depth-1, min_depth-2, -1):
-            out.write(whitespace_indent * (2*i+1) + '</li>\n' +
-                      whitespace_indent * (2*i) + '</ol>\n')
+        for i in xrange(curr_depth-1, min_depth-2, -1):
+            out.write(whitespace_indent * (2*i+1) + u'</li>\n' +
+                      whitespace_indent * (2*i) + u'</ol>\n')
 
     def _heading_formatter(self, match, fullmatch):
         depth, heading, anchor = self._parse_heading(match, fullmatch,
@@ -1575,7 +1588,7 @@ class HtmlFormatter(object):
         newlines in the wikidom will be preserved if `escape_newlines` is set.
         """
         # FIXME: compatibility code only for now
-        out = StringIO()
+        out = io.StringIO()
         Formatter(self.env, self.context).format(self.wikidom, out,
                                                  escape_newlines)
         return Markup(out.getvalue())
@@ -1603,7 +1616,7 @@ class InlineHtmlFormatter(object):
         have been emitted.
         """
         # FIXME: compatibility code only for now
-        out = StringIO()
+        out = io.StringIO()
         OneLinerFormatter(self.env, self.context).format(self.wikidom, out,
                                                          shorten)
         return Markup(out.getvalue())
@@ -1639,30 +1652,6 @@ def extract_link(env, context, wikidom):
 
 # pre-0.11 wiki text to Markup compatibility methods
 
-def wiki_to_html(wikitext, env, req, db=None,
-                 absurls=False, escape_newlines=False):
-    """deprecated in favor of format_to_html (will be removed in 1.0)"""
-    if not wikitext:
-        return Markup()
-    abs_ref, href = (req or env).abs_href, (req or env).href
-    from trac.web.chrome import web_context
-    context = web_context(req, absurls=absurls)
-    out = StringIO()
-    Formatter(env, context).format(wikitext, out, escape_newlines)
-    return Markup(out.getvalue())
-
-def wiki_to_oneliner(wikitext, env, db=None, shorten=False, absurls=False,
-                     req=None):
-    """:deprecated: in favor of format_to_oneliner (will be removed in 1.0)"""
-    if not wikitext:
-        return Markup()
-    abs_ref, href = (req or env).abs_href, (req or env).href
-    from trac.web.chrome import web_context
-    context = web_context(req, absurls=absurls)
-    out = StringIO()
-    OneLinerFormatter(env, context).format(wikitext, out, shorten)
-    return Markup(out.getvalue())
-
 def wiki_to_outline(wikitext, env, db=None,
                     absurls=False, max_depth=None, min_depth=None, req=None):
     """:deprecated: will be removed in 1.0 and replaced by something else"""
@@ -1671,6 +1660,6 @@ def wiki_to_outline(wikitext, env, db=None,
     abs_ref, href = (req or env).abs_href, (req or env).href
     from trac.web.chrome import web_context
     context = web_context(req, absurls=absurls)
-    out = StringIO()
+    out = io.StringIO()
     OutlineFormatter(env, context).format(wikitext, out, max_depth, min_depth)
     return Markup(out.getvalue())
