@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2020 Edgewall Software
+# Copyright (C) 2012-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -301,7 +301,9 @@ class AuthzPolicyTestCase(unittest.TestCase):
             * =
             """))
         authz_policy = AuthzPolicy(self.env)
+        authz_mtime = authz_policy.authz_mtime
         self.assertRaises(ConfigurationError, authz_policy.parse_authz)
+        self.assertEqual(authz_mtime, authz_policy.authz_mtime)
 
     # def test_parse_authz_duplicated_sections_raises(self):
     #     """ConfigurationError should be raised if the file has duplicate
