@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -18,14 +18,13 @@
 
 import posixpath
 
-from genshi.builder import tag
-
 from trac.config import ConfigSection
 from trac.core import *
 from trac.versioncontrol.api import NoSuchNode, RepositoryManager
 from trac.versioncontrol.web_ui.browser import IPropertyRenderer
 from trac.versioncontrol.web_ui.changeset import IPropertyDiffRenderer
 from trac.util import Ranges, to_ranges
+from trac.util.html import tag
 from trac.util.translation import _, tag_
 from trac.web.chrome import chrome_resource_path
 from tracopt.versioncontrol.svn.svn_fs import _path_within_scope
@@ -98,9 +97,9 @@ class SubversionPropertyRenderer(Component):
             for dummykey, value in self.svn_externals_section.options():
                 value = value.split()
                 if len(value) != 2:
-                    self.log.warn("svn:externals entry %s doesn't contain "
-                                  "a space-separated key value pair, "
-                                  "skipping.", dummykey)
+                    self.log.warning("svn:externals entry %s doesn't contain "
+                                     "a space-separated key value pair, "
+                                     "skipping.", dummykey)
                     continue
                 key, value = value
                 self._externals_map[key] = value.replace('%', '%%') \
